﻿using System;

using Command.Units;
using Command.Buildings;
using Command.Buildings.GDI;
using Command.Buildings.NOD;
using Command.Buildings.Upgrades;
using Command.Hardware;
using Command.Screens;
using Command.Commands;

namespace Command
{
    class Program
    {
        static void Main(string[] args)
        {
            #region Observer/Event Pattern Stuff
            /*
            GameUnit exampleUnit = new Soldier();

            BattleMapDisplay battleMap = new BattleMapDisplay(exampleUnit);
            OverviewDisplay overviewMap = new OverviewDisplay(exampleUnit);

            exampleUnit.Move(new Point(10, 10));
            exampleUnit.Move(new Point(20, 20));
            */
            #endregion

            #region Decorator Pattern Stuff
            IBuilding gdiBarracks = new Barracks();
            Console.WriteLine(string.Format("{0} needs {1} power.",
                                            gdiBarracks.Description,
                                            gdiBarracks.Power));

            IBuilding buildingWithOneTurret = new BuildingTurret(new Barracks());
            Console.WriteLine(string.Format("{0} needs {1} power.", 
                                            buildingWithOneTurret.Description, 
                                            buildingWithOneTurret.Power));

            IBuilding aRealFortress = new BuildingTurret(
                                        new BuildingTurret(
                                            new BuildingSAM(
                                                new BuildingLaser(
                                                    new Barracks()
                                                    )
                                                    )
                                                    )
                                                    );

            Console.WriteLine(string.Format("{0} needs {1} power.", 
                                            aRealFortress.Description, 
                                            aRealFortress.Power));

            Console.WriteLine();
            #endregion

            #region Factory Method Stuff
            InfantryFactory barracks = new Barracks();
            InfantryFactory handOfNod = new HandOfNod();

            GameUnit eliteUnitGDI = barracks.BuildUnit(UnitType.EliteUnit);
            Console.WriteLine("Built a {0}, working for {1}, set to {2}.", 
                eliteUnitGDI.Description, 
                eliteUnitGDI.Side, 
                eliteUnitGDI.Behaviour);

            GameUnit eliteUnitNOD = handOfNod.BuildUnit(UnitType.EliteUnit);
            Console.WriteLine("Built a {0}, working for {1}, set to {2}.", 
                eliteUnitNOD.Description, 
                eliteUnitNOD.Side, 
                eliteUnitNOD.Behaviour);

            Console.WriteLine();
            #endregion

            #region Singleton Pattern Stuff
            //GraphicsCard gfxCard = GraphicsCard.Instance;
            //GraphicsCardGeneric gfxCardGeneric = GraphicsCardGeneric.Instance;
            Console.WriteLine(GraphicsCard.Instance.Hello());
            Console.WriteLine(GraphicsCardGeneric.Instance.Hello());
            Console.WriteLine();
            #endregion

            #region Command Pattern Stuff
            // Our screen has some buttons in the right hand corner
            // Depending on the selected unit, it has different actions
            // Additionally, we want to have some recall last order functionality
            GameControls uiElement = new GameControls();

            // A soldier was selected, map the correct buttons
            GameUnit gdiSoldier = barracks.BuildUnit(UnitType.Soldier);
            ICommand stressSoldier = new StressUnit(gdiSoldier);
            ICommand relaxSoldier = new RelaxUnit(gdiSoldier);
            ICommand killSoldier = new SuicideUnit(gdiSoldier);
            uiElement.SetButton(0, stressSoldier);
            uiElement.SetButton(1, relaxSoldier);
            uiElement.SetButton(2, killSoldier);

            uiElement.ButtonPressed(0);
            uiElement.ButtonPressed(1);
            uiElement.UndoButtonPressed();

            uiElement.ButtonPressed(2);

            Console.WriteLine();
            // A building was selected, map the correct buttons
            ICommand powerDownBuilding = new PowerDownBuilding(aRealFortress);
            ICommand powerUpBuilding = new PowerUpBuilding(aRealFortress);
            uiElement.SetButton(0, powerDownBuilding);
            uiElement.SetButton(1, powerUpBuilding);

            uiElement.ButtonPressed(0);
            uiElement.ButtonPressed(1);
            uiElement.UndoButtonPressed();
            #endregion
        }
    }
}